@@ -39,15 +39,17 @@ class CameraWidget(QtWidgets.QMainWindow):
     REFRESH_START_STOP_PERIOD = 500 # how often we update settings with Tango
 
     # ----------------------------------------------------------------------
-    def __init__(self, parent, settings, my_name):
+    def __init__(self, parent, my_name):
         """
         """
         super(CameraWidget, self).__init__(parent)
 
-        self.settings = settings
+        self.settings = parent.settings
         self.camera_name = my_name
         self._parent = parent
 
+        self.hist_lock = QtCore.QMutex()
+
         self.camera_device = DataSource2D(self)
         state, msg = self.camera_device.new_device_proxy(self.camera_name, self._parent.auto_screen_action.isChecked())
         if not state:
@@ -65,15 +67,6 @@ class CameraWidget(QtWidgets.QMainWindow):
         self._refresh_run_stop_timer.timeout.connect(self._refresh_run_stop)
         self._refresh_run_stop_timer.start(self.REFRESH_START_STOP_PERIOD)
 
-    # ----------------------------------------------------------------------
-    def block_hist_signals(self, flag):
-        """
-            during the picture update in Frame viewer we need to disconnect histogram signals in Setting
-        :param flag: bool
-        :return:
-        """
-        self._settings_widget.block_hist_signals(flag)
-
     # ----------------------------------------------------------------------
     def _start_stop_live_mode(self):
         """
@@ -253,6 +246,10 @@ class CameraWidget(QtWidgets.QMainWindow):
         self._saveAsciiAction.triggered.connect(partial(self._frame_viewer.save_to_file, fmt="csv"))
         self._saveNumpyAction.triggered.connect(partial(self._frame_viewer.save_to_file, fmt="npy"))
 
+    # ----------------------------------------------------------------------
+    def block_hist_signals(self):
+        return self._settings_widget.block_hist_signals()
+
     # ----------------------------------------------------------------------
     def _add_dock(self, WidgetClass, label, *args, **kwargs):
 
