@@ -1,23 +1,21 @@
 # Created by matveyev at 22.02.2022
 
 import logging
-import os
-import shutil
-from pathlib import Path
 
 from PyQt5 import QtWidgets, QtGui, QtCore
 
+from petra_camera.utils.functions import get_save_path
 from petra_camera.main_window import APP_NAME
 from petra_camera.widgets.camera_settings import CameraSettings
 from petra_camera.gui.SettingsDialog_ui import Ui_SettingsDialog
 
-WIDGET_NAME = 'ProgramSetup'
-
 logger = logging.getLogger(APP_NAME)
 
 
 class ProgramSetup(QtWidgets.QDialog):
 
+    WIDGET_NAME = 'ProgramSetup'
+
     # ----------------------------------------------------------------------
     def __init__(self, main_window):
         """
@@ -38,9 +36,6 @@ class ProgramSetup(QtWidgets.QDialog):
         self._ui.tb_cameras.setCornerWidget(self.btn_add_camera, QtCore.Qt.TopRightCorner)
         self._ui.tb_cameras.cornerWidget().setMinimumSize(self.btn_add_camera.sizeHint())
 
-        # self._ui.cmd_sav_profile.clicked.connect(self._save_settings)
-        # self._ui.cmd_load_profile.clicked.connect(self._load_settings)
-
         self._ui.cmd_save_folder.clicked.connect(self._new_save_folder)
         self._ui.cmd_reset_settings.clicked.connect(self._reset_settings)
 
@@ -64,7 +59,7 @@ class ProgramSetup(QtWidgets.QDialog):
         if ok == QtWidgets.QMessageBox.Ok:
             self._main_window.reset_settings()
 
-            QtCore.QSettings(APP_NAME).setValue("{}/geometry".format(WIDGET_NAME), self.saveGeometry())
+            QtCore.QSettings(APP_NAME).setValue("{}/geometry".format(self.WIDGET_NAME), self.saveGeometry())
 
             super(ProgramSetup, self).accept()
 
@@ -78,7 +73,7 @@ class ProgramSetup(QtWidgets.QDialog):
     # ----------------------------------------------------------------------
     def _display_settings(self):
 
-        self._ui.le_save_path.setText(self._settings.option("save_folder", "default"))
+        self._ui.le_save_path.setText(get_save_path(self._settings))
 
         self._ui.cmd_roi_frame_color.setStyleSheet("QPushButton {background-color: " +
                                                    f"{self._settings.option('roi', 'fr_color')}" + ";}")
@@ -135,6 +130,20 @@ class ProgramSetup(QtWidgets.QDialog):
         self._ui.tb_cameras.setCurrentIndex(self._ui.tb_cameras.count() - 1)
         self._last_camera_id += 1
 
+    # ----------------------------------------------------------------------
+    def name_accepted(self, camera_id, new_name):
+        existing_names = []
+        for ind in range(self._ui.tb_cameras.count()):
+            if self._ui.tb_cameras.widget(ind).my_id != camera_id:
+                existing_names.append(self._ui.tb_cameras.widget(ind).my_name)
+
+        while new_name in existing_names:
+            new_name, accepted = QtWidgets.QInputDialog.getText(self, 'Enter an unique name!', 'Name: ')
+            if not accepted:
+                return None
+
+        return new_name
+
     # ----------------------------------------------------------------------
     def _delete_camera(self, id_to_delete):
         tab_to_delete = None
@@ -146,10 +155,10 @@ class ProgramSetup(QtWidgets.QDialog):
             self._ui.tb_cameras.removeTab(tab_to_delete)
 
     # ----------------------------------------------------------------------
-    def _new_name(self, id, new_name):
+    def _new_name(self, camera_id, new_name):
         tab_to_rename = None
         for ind in range(self._ui.tb_cameras.count()):
-            if self._ui.tb_cameras.widget(ind).my_id == id:
+            if self._ui.tb_cameras.widget(ind).my_id == camera_id:
                 tab_to_rename = ind
 
         if tab_to_rename is not None:
@@ -176,7 +185,7 @@ class ProgramSetup(QtWidgets.QDialog):
     def _apply_settings(self):
         general_options = []
 
-        general_options.append(("save_folder", (("default", self._ui.le_save_path.text()),)))
+        general_options.append(("save_folder", (("path", self._ui.le_save_path.text()),)))
 
         font = self._ui.cmd_roi_label_font.font()
         general_options.append(('roi', (('font', font.family() + ',' + str(font.pointSize())),
@@ -210,13 +219,13 @@ class ProgramSetup(QtWidgets.QDialog):
 
         self._apply_settings()
 
-        QtCore.QSettings(APP_NAME).setValue("{}/geometry".format(WIDGET_NAME), self.saveGeometry())
+        QtCore.QSettings(APP_NAME).setValue("{}/geometry".format(self.WIDGET_NAME), self.saveGeometry())
 
         super(ProgramSetup, self).accept()
 
     # ----------------------------------------------------------------------
     def reject(self):
 
-        QtCore.QSettings(APP_NAME).setValue("{}/geometry".format(WIDGET_NAME), self.saveGeometry())
+        QtCore.QSettings(APP_NAME).setValue("{}/geometry".format(self.WIDGET_NAME), self.saveGeometry())
 
         super(ProgramSetup, self).reject()
\ No newline at end of file
