@@ -14,21 +14,19 @@ import subprocess
 import logging
 import time
 
-import pyqtgraph as pg
-from packaging import version
-
 from PyQt5 import QtCore, QtWidgets
 from distutils.util import strtobool
+from contextlib import contextmanager
 
+from petra_camera.utils.functions import get_save_path
 from petra_camera.utils.errors import report_error
 from petra_camera.widgets.base_widget import BaseWidget
 from petra_camera.main_window import APP_NAME
 from petra_camera.gui.SettingsWidget_ui import Ui_SettingsWidget
+from petra_camera.external.histogramWidget import HistogramHLUTWidget
 
 from petra_camera.utils.functions import refresh_combo_box
 
-WIDGET_NAME = 'SettingsWidget'
-
 logger = logging.getLogger(APP_NAME)
 
 
@@ -36,6 +34,9 @@ logger = logging.getLogger(APP_NAME)
 class SettingsWidget(BaseWidget):
     """
     """
+
+    WIDGET_NAME = 'SettingsWidget'
+
     PARAMS_EDITOR = "atkpanel"
 
     # ----------------------------------------------------------------------
@@ -48,22 +49,14 @@ class SettingsWidget(BaseWidget):
         self._ui.setupUi(self)
         self._ui.gb_screen_motor.setVisible(self._camera_device.has_motor())
 
-        # picture histogram
-        if version.parse(pg.__version__) >= version.parse('0.12.0'):
-            self.hist = pg.HistogramLUTWidget(self, orientation='horizontal')
-        else:
-            self.hist = pg.HistogramLUTWidget(self)
+        self.hist = HistogramHLUTWidget(self)
+        self._hist_mutex = QtCore.QMutex()
 
         self.hist.setBackground('w')
         self._ui.vl_levels.addWidget(self.hist, 1)
 
-        self.hist.scene().sigMouseClicked.connect(self._hist_mouse_clicked)
-        self.hist.item.sigLevelChangeFinished.connect(self.new_levels)
-        self.hist.item.sigLookupTableChanged.connect(self.new_levels)
-        self.hist.items()[-1].setMenuEnabled(False) # TODO better way
-
         # to prevent double code run
-        self._my_mutex = QtCore.QMutex()
+        self._settings_mutex = QtCore.QMutex()
 
         self._start_settings_read = threading.Event()
         self._stop_settings_reader = threading.Event()
@@ -75,7 +68,13 @@ class SettingsWidget(BaseWidget):
 
         self._load_camera_settings()
 
+        self.hist.scene().sigMouseClicked.connect(self._hist_mouse_clicked)
+        self.hist.item.sigLevelChangeFinished.connect(self.new_levels)
+        self.hist.item.sigLookupTableChanged.connect(self.new_levels)
+        self.hist.items()[-1].setMenuEnabled(False) # TODO better way
+
         # setup all signals
+
         self._ui.tbAllParams.clicked.connect(self._edit_all_params)
 
         for ui in ['exposure', 'gain', 'FPS', 'reduce']:
@@ -105,6 +104,17 @@ class SettingsWidget(BaseWidget):
         self._ui.chk_background.clicked.connect(lambda state: self._camera_device.save_settings('background', state))
         self._ui.dsb_sigmas.valueChanged.connect(lambda value: self._camera_device.save_settings('background_sigmas', value))
 
+    # ----------------------------------------------------------------------
+    @contextmanager
+    def block_hist_signals(self):
+        self.hist.item.sigLevelChangeFinished.disconnect()
+        self.hist.item.sigLookupTableChanged.disconnect()
+
+        yield
+
+        self.hist.item.sigLevelChangeFinished.connect(self.new_levels)
+        self.hist.item.sigLookupTableChanged.connect(self.new_levels)
+
     # ----------------------------------------------------------------------
     def refresh_view(self, force_read=False):
         """
@@ -112,7 +122,7 @@ class SettingsWidget(BaseWidget):
         :return:
         """
 
-        with QtCore.QMutexLocker(self._my_mutex):
+        with QtCore.QMutexLocker(self._settings_mutex):
             self._block_signals(True)
 
             self._ui.chk_auto_levels.setChecked(self._camera_device.levels['auto_levels'])
@@ -144,7 +154,7 @@ class SettingsWidget(BaseWidget):
     # ----------------------------------------------------------------------
     def display_tango_settings(self):
 
-        with QtCore.QMutexLocker(self._my_mutex):
+        with QtCore.QMutexLocker(self._settings_mutex):
             self.blockSignals(True)
 
             if not self._ui.sb_exposure.hasFocus():
@@ -210,21 +220,8 @@ class SettingsWidget(BaseWidget):
         :param frame_view: picture form Frame viewer
         :return:
         """
-        self.hist.item.setImageItem(frame_view)
-
-    # ----------------------------------------------------------------------
-    def block_hist_signals(self, flag):
-        """
-
-        :param flag: bool
-        :return: None
-        """
-        if flag:
-            self.hist.item.sigLevelChangeFinished.disconnect()
-            self.hist.item.sigLookupTableChanged.disconnect()
-        else:
-            self.hist.item.sigLevelChangeFinished.connect(self.new_levels)
-            self.hist.item.sigLookupTableChanged.connect(self.new_levels)
+        with self.block_hist_signals():
+            self.hist.item.setImageItem(frame_view)
 
     # ----------------------------------------------------------------------
     def _switch_auto_levels(self, state):
@@ -247,14 +244,14 @@ class SettingsWidget(BaseWidget):
             self._ui.chk_auto_levels.setChecked(True)
 
     # ----------------------------------------------------------------------
-    def new_levels(self, lut_item):
+    def new_levels(self, lut_item, auto_levels=False):
         """
         slot for sigLookupTableChanged signal
         :param lut_item:
         :return:
         """
         lut = lut_item.saveState()
-        lut['auto_levels'] = self._ui.chk_auto_levels.isChecked()
+        lut['auto_levels'] = auto_levels
 
         self._camera_device.level_setting_change(lut)
 
@@ -275,7 +272,7 @@ class SettingsWidget(BaseWidget):
             lev[0] = float(value)
 
         self.hist.item.setLevels(lev[0], lev[1])
-        self.new_levels(self.hist.item)
+        self.new_levels(self.hist.item, self._ui.chk_auto_levels.isChecked())
 
     # ----------------------------------------------------------------------
     def _new_level_mode(self, button):
@@ -292,7 +289,7 @@ class SettingsWidget(BaseWidget):
     # ----------------------------------------------------------------------
     def _save_dark_image(self):
         fileName, _ = QtWidgets.QFileDialog.getSaveFileName(self, "Save dark image",
-                                                            self._settings.option("save_folder", "default"),
+                                                            get_save_path(self._settings),
                                                             "Numpy files (*.npy)")
         if fileName:
             self._camera_device.save_dark_image(fileName)
@@ -300,7 +297,7 @@ class SettingsWidget(BaseWidget):
     # ----------------------------------------------------------------------
     def _load_dark_image(self):
         file_name, _ = QtWidgets.QFileDialog.getOpenFileName(self, "Load dark image",
-                                                             self._settings.option("save_folder", "default"),
+                                                             get_save_path(self._settings),
                                                              "Numpy files (*.npy)")
         if file_name:
             self._camera_device.load_dark_image(file_name)
@@ -361,6 +358,10 @@ class SettingsWidget(BaseWidget):
         self._ui.sb_exposure.blockSignals(flag)
         self._ui.sb_gain.blockSignals(flag)
 
+        self._ui.chk_auto_levels.blockSignals(flag)
+        self._ui.sb_min_level.blockSignals(flag)
+        self._ui.sb_max_level.blockSignals(flag)
+
         self._ui.cmb_path.blockSignals(flag)
 
         self._ui.rb_lin_level.blockSignals(flag)
@@ -377,7 +378,7 @@ class SettingsWidget(BaseWidget):
             for layout in ['exposure', 'folder', 'FPS', 'source', 'background']:
                 getattr(self._ui, 'frame_{}'.format(layout)).setVisible(layout in self._camera_device.visible_layouts())
 
-            with QtCore.QMutexLocker(self._my_mutex):
+            with QtCore.QMutexLocker(self._settings_mutex):
 
                 self._ui.cmb_path.clear()
                 possible_folders = self._camera_device.get_settings('possible_folders', str)
@@ -397,9 +398,8 @@ class SettingsWidget(BaseWidget):
                 else:
                     self._ui.cmb_source.setEnabled(False)
 
-                self.block_hist_signals(True)
+                auto_levels = self._camera_device.levels['auto_levels']
                 self.hist.item.restoreState(self._camera_device.levels)
-                self.block_hist_signals(False)
 
             self.refresh_view(True)
 
@@ -426,7 +426,7 @@ class SettingsWidget(BaseWidget):
 
         state = False
         try:
-            state = strtobool(settings.value(f"{WIDGET_NAME}_{camera_name}/AdditionalSettings"))
+            state = strtobool(settings.value(f"{self.WIDGET_NAME}_{camera_name}/AdditionalSettings"))
         except:
             pass
 
@@ -444,7 +444,7 @@ class SettingsWidget(BaseWidget):
         super(SettingsWidget, self).save_ui_settings(camera_name)
 
         settings = QtCore.QSettings(APP_NAME)
-        settings.setValue(f"{WIDGET_NAME}_{camera_name}/AdditionalSettings", self._ui.chk_additional_settings.isChecked())
+        settings.setValue(f"{self.WIDGET_NAME}_{camera_name}/AdditionalSettings", self._ui.chk_additional_settings.isChecked())
 
     # ----------------------------------------------------------------------
     def close(self):
